##########################################################################
# Copyright 2009 Carlos Ribeiro
#
# This file is part of Radio Tray
#
# Radio Tray is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 1 of the License, or
# (at your option) any later version.
#
# Radio Tray is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Radio Tray.  If not, see <http://www.gnu.org/licenses/>.
#
##########################################################################
import sys
from urllib.request import Request as UrlRequest
from urllib.request import urlopen as urlUrlopen
import logging
from common import USER_AGENT

class RamPlaylistDecoder:
    def __init__(self):
        self.log = logging.getLogger('upmpdcli')


    def isStreamValid(self, contentType, firstBytes):
        if 'audio/x-pn-realaudio' in contentType or \
           'audio/vnd.rn-realaudio' in contentType:
            self.log.info('Stream is readable by RAM Playlist Decoder')
            return True
        else:
            return False


    def extractPlaylist(self,  url):
        self.log.info('RAM: Downloading playlist...')
        req = UrlRequest(url)
        req.add_header('User-Agent', USER_AGENT)
        f = urUrlopen(req)
        str = f.read()
        f.close()

        self.log.info('RAM Playlist downloaded, decoding...')

        lines = str.splitlines()
        playlist = []
        for line in lines:
            if len(line) > 0 and not line.startswith(b"#"):
                tmp = line.strip()
                if len(tmp) > 0:
                    playlist.append(line.strip())

        return playlist



# Copyright (C) 2023 Giovanni Fulco
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Callable
from track_adapter import TrackAdapter
from tidalapi.session import Session as TidalSession
from tidalapi.media import Track as TidalTrack
from tidalapi.album import Album as TidalAlbum
import cmdtalkplugin
import tidal_util


# Func name to method mapper
dispatcher = cmdtalkplugin.Dispatch()
# Pipe message handler
msgproc = cmdtalkplugin.Processor(dispatcher)


class TidalTrackAdapter(TrackAdapter):

    def __init__(
            self,
            tidal_session: TidalSession,
            track: TidalTrack,
            album_retriever: Callable[[TidalSession, str], TidalAlbum]):
        self._tidal_session: TidalSession = tidal_session
        self._track: TidalTrack = track
        self._album_retriever: Callable[[TidalSession, str], TidalAlbum] = album_retriever
        self._album: TidalAlbum = None
        self._album_load_failed: bool = False
        self._stream = None
        self._stream_load_failed: bool = False
        self._track_image_url: str = tidal_util.get_image_url(obj=track)

    def __get_album(self):
        if not self._album_load_failed and not self._album:
            try:
                self._album = self._album_retriever(self._tidal_session, self._track.album.id)
            except Exception as ex:
                msgproc.log(f"Cannot get album for id [{self._track.album.id}] due to [{type(ex)}] [{ex}]")
                self._album_load_failed = True
                raise ex
        return self._album

    def __load_stream(self):
        if not self._stream_load_failed and not self._stream:
            try:
                self._stream = self._track.get_stream()
            except Exception as ex:
                msgproc.log(f"Cannot get stream for track id [{self._track.id}] due to [{type(ex)}] [{ex}]")
                self._stream_load_failed = True
                raise ex
        return self._stream

    def get_id(self) -> str:
        return str(self._track.id)

    def get_name(self) -> str:
        return self._track.name

    def get_duration(self) -> int:
        return self._track.duration

    def get_track_num(self) -> int:
        return self._track.track_num

    def get_volume_num(self) -> int:
        return self._track.volume_num

    def get_album_num_volumes(self) -> int:
        album: TidalAlbum = self.__get_album()
        try:
            return album.num_volumes
        except Exception:
            # no info for num_volumes
            return None

    def get_album_track_count(self) -> int:
        return self.__get_album().num_tracks

    def get_album_name(self) -> str:
        return self._track.album.name

    def get_album_id(self) -> str:
        return self._track.album.id

    def get_album_artist_name(self) -> str:
        return self._track.album.artist.name

    def get_image_url(self) -> str:
        return self._track_image_url

    def explicit(self) -> bool:
        return self._track.explicit

    def get_artist_name(self) -> str:
        return self._track.artist.name

    def get_bit_depth(self) -> int:
        return self.__load_stream().bit_depth

    def get_sample_rate(self) -> int:
        return self.__load_stream().sample_rate

    def get_audio_quality(self) -> str:
        return self.__load_stream().audio_quality



# Copyright (C) 2024 Giovanni Fulco
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from tidalapi import Quality as TidalQuality


class StreamingInfo:

    def __init__(self):
        self._url: str = None
        self._mimetype: str = None
        self._codec: any = None
        self._sample_rate: int = None
        self._bit_depth: int = None
        self._audio_quality: TidalQuality = None
        self._audio_mode: str = None

    @property
    def url(self) -> str:
        return self._url

    @url.setter
    def url(self, value: str):
        self._url: str = value

    @property
    def mimetype(self) -> str:
        return self._mimetype

    @mimetype.setter
    def mimetype(self, value: str):
        self._mimetype: str = value

    @property
    def codecs(self) -> any:
        return self._codec

    @codecs.setter
    def codecs(self, value: any):
        self._codec = value

    @property
    def sample_rate(self) -> int:
        return self._sample_rate

    @sample_rate.setter
    def sample_rate(self, value: int):
        self._sample_rate = value

    @property
    def bit_depth(self) -> int:
        return self._bit_depth

    @bit_depth.setter
    def bit_depth(self, value: int):
        self._bit_depth = value

    @property
    def audio_quality(self) -> TidalQuality:
        return self._audio_quality

    @audio_quality.setter
    def audio_quality(self, value: TidalQuality):
        self._audio_quality = value

    @property
    def audio_mode(self) -> str:
        return self._audio_mode

    @audio_mode.setter
    def audio_mode(self, value: str):
        self._audio_mode = value
